@@ -31,20 +31,27 @@ These tests are run automatically with 'setup.py test', but can also be run
 manually.
 """
 
+from os import listdir
+from os.path import dirname
 import sys
 import unittest
 
 
-names = ['blob', 'commit', 'config', 'diff', 'index', 'refs', 'remote',
-         'repository', 'revwalk', 'signature', 'status', 'tag', 'tree',
-         'treebuilder', 'note']
-
 def test_suite():
     # Sometimes importing pygit2 fails, we try this first to get an
-    # informative traceback
+    # informative traceback.
     import pygit2
+
+    # Build the list of modules
+    modules = []
+    for name in listdir(dirname(__file__)):
+        if name.startswith('test_') and name.endswith('.py'):
+            module = 'test.%s' % name[:-3]
+            # Check the module imports correctly, have a nice error otherwise
+            __import__(module)
+            modules.append(module)
+
     # Go
-    modules = ['test.test_%s' % n for n in names]
     return unittest.defaultTestLoader.loadTestsFromNames(modules)
 
 
