@@ -29,6 +29,7 @@
 
 from __future__ import absolute_import
 from __future__ import unicode_literals
+from os.path import dirname, join
 import unittest
 
 import pygit2
@@ -49,7 +50,7 @@ class BlobTest(utils.RepoTestCase):
     def test_read_blob(self):
         blob = self.repo[BLOB_SHA]
         self.assertEqual(blob.hex, BLOB_SHA)
-        sha = utils.oid_to_hex(blob.oid)
+        sha = blob.oid.hex
         self.assertEqual(sha, BLOB_SHA)
         self.assertTrue(isinstance(blob, pygit2.Blob))
         self.assertEqual(pygit2.GIT_OBJ_BLOB, blob.type)
@@ -66,17 +67,16 @@ class BlobTest(utils.RepoTestCase):
 
         self.assertEqual(blob_oid, blob.oid)
         self.assertEqual(
-          utils.gen_blob_sha1(BLOB_NEW_CONTENT),
-          utils.oid_to_hex(blob_oid)
-        )
+            utils.gen_blob_sha1(BLOB_NEW_CONTENT),
+            blob_oid.hex)
 
         self.assertEqual(BLOB_NEW_CONTENT, blob.data)
         self.assertEqual(len(BLOB_NEW_CONTENT), blob.size)
         self.assertEqual(BLOB_NEW_CONTENT, blob.read_raw())
 
-    def test_create_blob_fromfile(self):
+    def test_create_blob_fromworkdir(self):
 
-        blob_oid = self.repo.create_blob_fromfile("bye.txt")
+        blob_oid = self.repo.create_blob_fromworkdir("bye.txt")
         blob = self.repo[blob_oid]
 
         self.assertTrue(isinstance(blob, pygit2.Blob))
@@ -84,13 +84,26 @@ class BlobTest(utils.RepoTestCase):
 
         self.assertEqual(blob_oid, blob.oid)
         self.assertEqual(
-          utils.gen_blob_sha1(BLOB_FILE_CONTENT),
-          utils.oid_to_hex(blob_oid)
-        )
+            utils.gen_blob_sha1(BLOB_FILE_CONTENT),
+            blob_oid.hex)
 
         self.assertEqual(BLOB_FILE_CONTENT, blob.data)
         self.assertEqual(len(BLOB_FILE_CONTENT), blob.size)
         self.assertEqual(BLOB_FILE_CONTENT, blob.read_raw())
 
+
+    def test_create_blob_outside_workdir(self):
+        path = join(dirname(__file__), 'data', self.repo_dir + '.tar')
+        self.assertRaises(KeyError, self.repo.create_blob_fromworkdir, path)
+
+
+    def test_create_blob_fromdisk(self):
+        path = join(dirname(__file__), 'data', self.repo_dir + '.tar')
+        blob_oid = self.repo.create_blob_fromdisk(path)
+        blob = self.repo[blob_oid]
+
+        self.assertTrue(isinstance(blob, pygit2.Blob))
+        self.assertEqual(pygit2.GIT_OBJ_BLOB, blob.type)
+
 if __name__ == '__main__':
     unittest.main()
