@@ -72,7 +72,7 @@ class TagTest(utils.BareRepoTestCase):
 
         self.assertEqual('3ee44658fd11660e828dfc96b9b5c5f38d5b49bb', tag.hex)
         self.assertEqual(name, tag.name)
-        self.assertEqual(target, utils.oid_to_hex(tag.target))
+        self.assertEqual(target, tag.target.hex)
         self.assertEqualSignature(tagger, tag.tagger)
         self.assertEqual(message, tag.message)
         self.assertEqual(name, self.repo[tag.hex].name)
